import React, { useEffect, useRef } from 'react';
import { elementContainsDomRect } from './elementContainsDomRect';
import { rangeToTextRects } from './rangeToTextRects';
import { domRectToSelectionRectangle } from './TextSelection';
const notNull = (value) => value !== null;
const normalizedFirefoxRange = (selection) => {
    const finalRange = selection.getRangeAt(selection.rangeCount - 1);
    const firstRange = selection.getRangeAt(0);
    const range = document.createRange();
    range.setStart(firstRange.startContainer, firstRange.startOffset);
    range.setEnd(finalRange.endContainer, finalRange.endOffset);
    return range;
};
const HandleTextSelection = ({ onSelect, onDeselect, children, }) => {
    const ref = useRef(null);
    const hasSelected = useRef(false);
    const deselectionHandler = () => {
        if (onDeselect && hasSelected.current) {
            onDeselect();
            hasSelected.current = false;
        }
    };
    useEffect(() => {
        document.addEventListener('selectionchange', deselectionHandler);
        return () => {
            document.removeEventListener('selectionchange', deselectionHandler);
        };
    }, []);
    const getSelection = () => {
        const selection = window.getSelection();
        if (!ref.current || !(selection === null || selection === void 0 ? void 0 : selection.toString().trim())) {
            return;
        }
        hasSelected.current = true;
        const regionElements = Array.from(ref.current.querySelectorAll('div[data-region-selector-id]'));
        const range = normalizedFirefoxRange(selection);
        const selectionRectangles = rangeToTextRects(range)
            .map(rectangle => {
            const region = regionElements.find(elementContainsDomRect(rectangle));
            if (!region) {
                return null;
            }
            return domRectToSelectionRectangle(rectangle, region);
        })
            .filter(notNull);
        onSelect({ text: selection.toString(), selectionRectangles });
    };
    return (React.createElement("div", { role: "none", ref: ref, onMouseDown: e => {
            var _a;
            if (!e.shiftKey) {
                (_a = window.getSelection()) === null || _a === void 0 ? void 0 : _a.removeAllRanges();
            }
        }, onMouseUp: getSelection }, children));
};
export { HandleTextSelection };
//# sourceMappingURL=HandleTextSelection.js.map